#!/usr/bin/env python3
# -----------------------------------------------------------------------------
# Programmer(s): David J. Gardner @ LLNL
# -----------------------------------------------------------------------------
# SUNDIALS Copyright Start
# Copyright (c) 2025-2026, Lawrence Livermore National Security,
# University of Maryland Baltimore County, and the SUNDIALS contributors.
# Copyright (c) 2013-2025, Lawrence Livermore National Security
# and Southern Methodist University.
# Copyright (c) 2002-2013, Lawrence Livermore National Security.
# All rights reserved.
#
# See the top-level LICENSE and NOTICE files for details.
#
# SPDX-License-Identifier: BSD-3-Clause
# SUNDIALS Copyright End
# -----------------------------------------------------------------------------
# matplotlib-based plotting script for cvsPendulum_dns.c example
# -----------------------------------------------------------------------------

# imports
import argparse
import numpy as np
import matplotlib.pyplot as plt

# command line options
parser = argparse.ArgumentParser(description="Plots cvsPendulum_dns output")
parser.add_argument("sfile", type=str, help="solution output file to read")

# parse inputs
args = parser.parse_args()

# read solution output file
data = np.loadtxt(args.sfile, dtype=np.double)

# extract times, positions, and velocities
t = data[:, 0]
x = data[:, 1]
y = data[:, 2]
vx = data[:, 3]
vy = data[:, 4]

# read reference solution output file
ref = np.loadtxt("cvsPendulum_dns_ref.txt", dtype=np.double)

# extract positions and velocities
xr = ref[:, 1]
yr = ref[:, 2]
vxr = ref[:, 3]
vyr = ref[:, 4]

# lower half of unit circle
tt = np.linspace(np.pi, 2 * np.pi, 10000)
xt = np.cos(tt)
yt = np.sin(tt)

# plot solution in xy plane
fig, ax = plt.subplots()
ax.axhline(y=0, color="black", linestyle="--", label=None)
ax.axvline(x=0, color="black", linestyle="--", label=None)
plt.plot(xt, yt, color="black", linestyle="--", label=None)
plt.scatter(x, y, color="red", label="comp")
plt.scatter(xr, yr, color="blue", label="ref")

plt.xlabel("x")
plt.ylabel("y")
plt.title("Pendulum")
ax.set_aspect("equal")
plt.legend(loc="lower right")

# plot position over time
fig, ax = plt.subplots()
ax.axhline(y=0, color="black", linestyle="--")
plt.plot(t, x, label="x")
plt.plot(t, y, label="y")

plt.xlabel("t")
plt.ylabel("position")
plt.title("Pendulum Position")
plt.legend()

# plot velocity over time
fig, ax = plt.subplots()
ax.axhline(y=0, color="black", linestyle="--")
plt.plot(t, vx, label="$v_x$")
plt.plot(t, vy, label="$v_y$")

plt.xlabel("t")
plt.ylabel("velocity")
plt.title("Pendulum Velocity")
plt.legend()

# display plots
plt.show()

##### end of script #####



#!/usr/bin/env python3
# ----------------------------------------------------------------
# Programmer(s): Daniel R. Reynolds @ UMBC
# ----------------------------------------------------------------
# SUNDIALS Copyright Start
# Copyright (c) 2025-2026, Lawrence Livermore National Security,
# University of Maryland Baltimore County, and the SUNDIALS contributors.
# Copyright (c) 2013-2025, Lawrence Livermore National Security
# and Southern Methodist University.
# Copyright (c) 2002-2013, Lawrence Livermore National Security.
# All rights reserved.
#
# See the top-level LICENSE and NOTICE files for details.
#
# SPDX-License-Identifier: BSD-3-Clause
# SUNDIALS Copyright End
# ----------------------------------------------------------------
# matplotlib-based plotting script for heat1D.c example

# imports
import sys
import pylab as plt
import numpy as np

# load mesh data file as list of NumPy arrays
inp = open("heat_mesh.txt").readlines()
mesh = []
for line in inp:
    mesh.append(np.array(str.split(line), dtype=np.double))


# load solution data file as list of NumPy arrays
inp = open("heat1D.txt").readlines()
data = []
for line in inp:
    data.append(np.array(str.split(line), dtype=np.double))

# determine number of time steps
nt = len(mesh)
nt2 = len(data)
if nt != nt2:
    sys.exit(
        "plot_heat1D_adapt.py error: data and mesh files have different numbers of time steps"
    )

# determine minimum/maximum temperature
mintemp = 0.0
maxtemp = 0.0
for tstep in range(nt):
    mx = data[tstep].max()
    if mx > maxtemp:
        maxtemp = mx
    mn = data[tstep].min()
    if mn < mintemp:
        mintemp = mn
if maxtemp > 0.0:
    maxtemp *= 1.1
else:
    maxtemp *= 0.9
if mintemp > 0.0:
    mintemp *= 0.9
else:
    mintemp *= 1.1


# generate plots of results
for tstep in range(nt):

    # set string constants for output plots, current time, mesh size
    pname = "heat1d." + repr(tstep).zfill(3) + ".png"
    tstr = repr(tstep)
    nxstr = repr(len(data[tstep]))

    # plot current solution and save to disk
    plt.figure(1)
    plt.plot(mesh[tstep], data[tstep], "-o")
    plt.xlabel("x")
    plt.ylabel("solution")
    plt.title("u(x) at output " + tstr + ", mesh = " + nxstr)
    plt.axis((0.0, 1.0, mintemp, maxtemp))
    plt.grid()
    plt.savefig(pname)
    plt.close()


##### end of script #####



#!/usr/bin/env python3
# -----------------------------------------------------------------------------
# Programmer(s): David J. Gardner @ LLNL
# -----------------------------------------------------------------------------
# SUNDIALS Copyright Start
# Copyright (c) 2025-2026, Lawrence Livermore National Security,
# University of Maryland Baltimore County, and the SUNDIALS contributors.
# Copyright (c) 2013-2025, Lawrence Livermore National Security
# and Southern Methodist University.
# Copyright (c) 2002-2013, Lawrence Livermore National Security.
# All rights reserved.
#
# See the top-level LICENSE and NOTICE files for details.
#
# SPDX-License-Identifier: BSD-3-Clause
# SUNDIALS Copyright End
# -----------------------------------------------------------------------------
# matplotlib-based plotting script for cvsPraticle_dns.c example
# -----------------------------------------------------------------------------

# imports
import argparse
import numpy as np
import matplotlib.pyplot as plt

# command line options
parser = argparse.ArgumentParser(description="Plots cvsPraticle_dns output")
parser.add_argument(
    "--sfile", type=str, default="cvsParticle_solution.txt", help="solution output file to read"
)
parser.add_argument(
    "--efile", type=str, default="cvsParticle_error.txt", help="error output file to read"
)
parser.add_argument(
    "--alpha", type=float, nargs=1, default=1.0, help="set a non-default alpha value"
)
parser.add_argument("--slim", type=float, nargs=2, help="x and y limits for solution plot")
parser.add_argument("--eylim", type=float, nargs=2, help="y limits for error plot")

# parse inputs
args = parser.parse_args()

# read solution output file
data = np.loadtxt(args.sfile, dtype=np.double)

# extract times and positions
t = data[:, 0]
x = data[:, 1]
y = data[:, 2]

# unit circle
tt = np.linspace(0, np.pi * 2, 10000)
xt = np.cos(tt)
yt = np.sin(tt)

# plot solution
fig, ax = plt.subplots()
plt.plot(xt, yt, color="black", linestyle="--")
plt.scatter(x, y, color="red")

if args.slim:
    plt.xlim((args.slim[0], args.slim[1]))
    plt.ylim((args.slim[0], args.slim[1]))

plt.xlabel("x")
plt.ylabel("y")
plt.title("Solution")
ax.set_aspect("equal")

# true solution
xt = np.cos(args.alpha * t)
yt = np.sin(args.alpha * t)

# plot solution
fig, ax = plt.subplots()
plt.plot(t, x, linestyle="-", label="x")
plt.plot(t, xt, linestyle="--", label="x true")
plt.plot(t, y, linestyle="-", label="y")
plt.plot(t, yt, linestyle="--", label="y true")

plt.xlabel("t")
plt.ylabel("position")
plt.title("Particle Position Over Time")
plt.legend(loc="lower right")

# read error output file
data = np.loadtxt(args.efile, dtype=np.double)

# extract times, position errors, and constraint error
t = data[:, 0]
xerr = np.absolute(data[:, 1])
yerr = np.absolute(data[:, 2])
cerr = np.absolute(data[:, 3])

# plot solution
fig, ax = plt.subplots()
plt.semilogy(t, xerr, label="x err")
plt.semilogy(t, yerr, label="y err")
plt.semilogy(t, cerr, label="c err")

if args.eylim:
    plt.ylim((args.eylim[0], args.eylim[1]))

plt.xlabel("time")
plt.ylabel("error")
plt.legend(loc="lower right")
plt.title("Error in position and constraint")
plt.grid()

# display plots
plt.show()

##### end of script #####
